@@ -1,30 +1,30 @@
-(* 
-   Copyright 2003 Savonet team
+(*
+  Copyright 2003-2004 The Savonet team
 
-   This file is part of Ocaml-vorbis.
-   
-   Ocaml-vorbis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   
-   Ocaml-vorbis is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with Ocaml-vorbis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  This file is part of Ocaml-vorbis.
+
+  Ocaml-vorbis is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  Ocaml-vorbis is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with Ocaml-vorbis; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
 (**
   Decode from or encode to the Ogg Vorbis compressed audio format; or get informations about an Ogg Vorbis file.
 
-  @author Samuel Mimram, Julien Cristau
+  @author Samuel Mimram, Julien Cristau, David Baelde
 *)
 
-(* $Id: vorbis.mli,v 1.31 2004/06/29 22:56:47 smimram Exp $ *)
+(* $Id: vorbis.mli,v 1.36 2005/04/15 15:32:07 smimram Exp $ *)
 
 (** {1 Exceptions} *)
 
@@ -76,13 +76,15 @@ type enc_params =
       enc_in_big_endian : bool; (** is the input data stored in big endian? *)
     }
 
-(** Create a new encoder. *)
+(** Create a new encoder. The second argument is a string containing the header frame. *)
 val create_encoder :
   ?title:string ->
   ?artist:string ->
   ?genre:string ->
   ?date:string ->
   ?album:string -> ?tracknum:string -> enc_params -> encoder * string
+
+(** Same as [create_encoder] but with option arguments. *)
 val create_encoder_opt :
   string option ->
   string option ->
@@ -91,7 +93,7 @@ val create_encoder_opt :
   string option -> string option -> enc_params -> encoder * string
 
 
-(** Encode a wav buffer into ogg. *)
+(** Encode a wav buffer into ogg. WARNING: the size of the input buffer should be less than or equal to 1024 bytes. *)
 val encode_buffer : encoder -> string -> string
 
 (** {1 Decoding} *)
@@ -112,7 +114,10 @@ type dec_file
 (** Open an ogg/vorbis file for decoding. Can raise: [Could_not_open_file] or [Not_a_vorbis_file]. *)
 val open_dec_file : string -> dec_params -> dec_file
 
-(** [decode dec_file buf ofs len] decodes [len] octets of wav and put them in [buf] starting at position [pos]. It can raise: [End_of_file] when the whole file has been decoded, [Hole_in_data], [Invalid_parameters] if all the data cannot fit in the buffer starting at the given position. *)
+(** Same as [open_dec_file] but uses a file descriptor instead of a file name. The file descriptor is closed by [close_dec_file]. *)
+val open_dec_fd : Unix.file_descr -> dec_params -> dec_file
+
+(** [decode dec_file buf ofs len] decodes [len] octets of wav and put them in [buf] starting at position [pos]. It can raise: [Unix.End_of_file] when the whole file has been decoded, [Hole_in_data], [Invalid_parameters] if all the data cannot fit in the buffer starting at the given position. *)
 val decode : dec_file -> string -> int -> int -> int
 
 (** Close an ogg/vorbis file opened for decoding. The file descriptor used for the matching [open_dec_file] is closed (I think). *)
@@ -134,7 +139,7 @@ val encoder_reset_opt :
 (** {1 Informations about files} *)
 
 (** Vorbis informations about a file. *)
-type info = { 
+type info = {
   vorbis_version : int; (** version of vorbis codec, must be 0 *)
   audio_channels : int; (** number of audio channels *)
   audio_sample_rate : int; (** samplerate in Hertz *)
@@ -172,18 +177,23 @@ end
 module Info : functor (Io : Iofile) ->
 sig
   (** Get the vorbis comments from a vorbis file.
-    The first string is the vendor string, the elements of the array are 
-    of the form (field name, field contents), where 'field name' is 
-    uppercase. 
+    The first string is the vendor string, the elements of the array are
+    of the form (field name, field contents), where 'field name' is
+    uppercase.
   *)
   val get_comments : string -> string * (string*string) array
-    
-  (** Get the information in the header of a vorbis file with a safe value of duration. *)
+
+  (**
+     Get the information in the header of a vorbis file with a safe
+     value of duration.
+     @raise Unix.End_of_file if the file is too small (no complete pages)
+  *)
   val get_info : string -> info
 
-  (** Same as get_info but uses a heuristic to find the duration of the vorbis file without reading every page header. *)
+  (** Same as get_info but uses a heuristic to find the duration of
+      the vorbis file without reading every page header. *)
   val get_heuristical_info : string -> info
-	
+
   (** Get the size of a given file (in bytes). *)
   val file_size : string -> int
 end
