@@ -1,21 +1,21 @@
-/* 
-   Copyright 2003 Savonet team
+/*
+   Copyright 2003-2004 The Savonet Team
    Some parts from oggenc/oggdec, (c) 2000-2002 Michael Smith
    Some parts from ices2 (c) 2001 Michael Smith <msmith@labyrinth.net.au>
    See COPYRIGHT file for details.
 
    This file is part of Ocaml-vorbis.
-   
+
    Ocaml-vorbis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    Ocaml-vorbis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with Ocaml-vorbis; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -24,10 +24,10 @@
 /**
  * Libvorbis bindings for OCaml.
  *
- * @author Samuel Mimram, Julien Cristau
+ * @author Samuel Mimram, Julien Cristau, David Baelde
  */
 
-/* $Id: vorbis_stubs.c,v 1.25 2004/06/29 22:56:47 smimram Exp $ */
+/* $Id: vorbis_stubs.c,v 1.34 2005/03/29 00:20:33 dbaelde Exp $ */
 
 /*
   TODO:
@@ -35,6 +35,7 @@
   - advanced encoder options
 */
 
+#define CAML_NAME_SPACE
 #include <caml/alloc.h>
 #include <caml/callback.h>
 #include <caml/fail.h>
@@ -61,7 +62,10 @@ struct myvorbis__encoder_t
   ogg_page og;
   ogg_packet op;
 
+  int channels;
   int in_channels;
+  int samplefreq;
+  int in_samplefreq;
   int in_samplesize;
   int in_bigendian;
   int serial;
@@ -69,7 +73,7 @@ struct myvorbis__encoder_t
 
 typedef struct myvorbis__encoder_t myvorbis_encoder_t;
 
-static myvorbis_encoder_t* ve_of_block(value block)
+inline static myvorbis_encoder_t* ve_of_block(value block)
 {
   return (myvorbis_encoder_t*)Field(block, 1);
 }
@@ -86,7 +90,7 @@ static void finalize_myvorbis_encoder_t(value v_e)
   free(ve);
 }
 
-static int int_of_option(value v)
+inline static int int_of_option(value v)
 {
   if(Is_block(v))
     return Int_val(Field(v, 0));
@@ -94,17 +98,7 @@ static int int_of_option(value v)
     return -1;
 }
 
-/*
-static double double_of_option(value v)
-{
-  if(Is_block(v))
-    return Double_val(Field(v, 0));
-  else
-    return -1;
-}
-*/
-
-static char* string_of_option(value v)
+inline static char* string_of_option(value v)
 {
   if(Is_block(v))
     return String_val(Field(v, 0));
@@ -112,15 +106,11 @@ static char* string_of_option(value v)
     return NULL;
 }
 
-static value copy_buffer(char const *b, int len)
-{
-  CAMLlocal1(res);
-  res = alloc_string(len);
-  memmove(String_val(res), b, len);
-  return res;
-}
+#define copy_buffer(b,len,dst) \
+  dst = caml_alloc_string(len); \
+  memmove(String_val(dst), b, len);
 
-static char* strncat_safe(char* dest, char* src, int pos, int len) {
+inline static char* strncat_safe(char* dest, char* src, int pos, int len) {
   dest = (char*)realloc((void*)dest, sizeof(char)*(pos+len));
   memcpy(dest+pos, src, len);
   return dest;
@@ -138,7 +128,7 @@ static void add_tag(vorbis_comment *vc, char *name, char *val)
       free(utf8);
     }
   else
-    raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), copy_string(val));
+    caml_raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), caml_copy_string(val));
 }
 
 static void build_comments(vorbis_comment *vc,
@@ -159,7 +149,7 @@ static void build_comments(vorbis_comment *vc,
     add_tag(vc, "album", album);
   if (tracknum)
     add_tag(vc, "tracknumber", tracknum);
-  add_tag(vc, "encoder", "ocaml-vorbis by the savonet team (http://savonet.sf.net)");
+  add_tag(vc, "encoder", "ocaml-vorbis by the savonet team (http://savonet.sf.net/)");
 }
 
 
@@ -202,12 +192,11 @@ CAMLprim value ocaml_vorbis_get_header_frame(myvorbis_encoder_t * ve, value titl
       }
   }
 
-  s = copy_buffer(ret, ret_len);
+  copy_buffer(ret, ret_len, s);
   free(ret);
   CAMLreturn(s);
 }
 
-/* TODO: check parameters consistency */
 CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value artist, value genre, value date, value album, value tracknum)
 {
   CAMLparam5(params, title, artist, genre, date);
@@ -224,6 +213,7 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
   int channels = Int_val(Field(params, 4));
   int sample_freq = int_of_option(Field(params, 5));
   int managed = Bool_val(Field(params, 6));
+
   int in_channels = Int_val(Field(params, 7));
   int in_samplefreq = Int_val(Field(params, 8));
   int in_samplesize = Int_val(Field(params, 9));
@@ -233,16 +223,21 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
   srand(time(NULL));
   serialno = rand();
 
-  /* TODO: other checks */
-  if(sample_freq < 8000 || sample_freq > 48000)
-    raise_constant(*caml_named_value("vorbis_exn_invalid_sample_freq"));
-  if(channels != 1 && channels != 2 )
-    raise_constant(*caml_named_value("vorbis_exn_invalid_channels"));
+  /* Frequency conversion is performed only for this simple cases */
+  if(in_samplefreq % sample_freq != 0)
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_sample_freq"));
+  if((channels != 1 && channels != 2) || (in_channels != 1 && in_channels != 2))
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_channels"));
+  if(in_samplesize != 8 && in_samplesize != 16)
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_samplesize"));
 
   ve = malloc(sizeof(myvorbis_encoder_t));
 
+  ve->channels = channels;
   ve->in_channels = in_channels;
   ve->in_samplesize = in_samplesize;
+  ve->samplefreq = sample_freq;
+  ve->in_samplefreq = in_samplefreq;
   ve->in_bigendian = in_bigendian;
   ve->serial = serialno;
 
@@ -258,15 +253,15 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
       if (vorbis_encode_setup_vbr(&ve->vi, channels, sample_freq, quality))
 	{
 	  vorbis_info_clear(&ve->vi);
-	  raise_constant(*caml_named_value("vorbis_exn_invalid_quality"));
+	  caml_raise_constant(*caml_named_value("vorbis_exn_invalid_quality"));
 	}
 
       /* Do we have optional hard quality restrictions? */
       if (max_bitrate > 0 || min_bitrate > 0)
 	{
 	  struct ovectl_ratemanage_arg ai;
-	  
-	  vorbis_encode_ctl(&ve->vi, OV_ECTL_RATEMANAGE_GET, &ai);  
+
+	  vorbis_encode_ctl(&ve->vi, OV_ECTL_RATEMANAGE_GET, &ai);
 	  ai.bitrate_hard_min = min_bitrate * 1000;
 	  ai.bitrate_hard_max = max_bitrate * 1000;
 	  ai.management_active = 1;
@@ -276,12 +271,12 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
   else
     {
       if (vorbis_encode_setup_managed(&ve->vi, channels, sample_freq,
-				      max_bitrate>0?max_bitrate*1000:-1, 
+				      max_bitrate>0?max_bitrate*1000:-1,
 				      bitrate*1000,
 				      min_bitrate>0?min_bitrate*1000:-1))
 	{
 	  vorbis_info_clear(&ve->vi);
-	  raise_constant(*caml_named_value("vorbis_exn_invalid_bitrate"));
+	  caml_raise_constant(*caml_named_value("vorbis_exn_invalid_parameters"));
 	}
     }
 
@@ -303,13 +298,13 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
 
   ogg_stream_init(&ve->os, serialno);
 
-  encoder = alloc_final(2, finalize_myvorbis_encoder_t, sizeof(myvorbis_encoder_t), 1024);
-  Field(encoder, 1) = (value)ve;
+  encoder = caml_alloc_final(2, finalize_myvorbis_encoder_t, sizeof(myvorbis_encoder_t), 1024);
+  Store_field(encoder, 1, (value)ve);
 
-  block = alloc_tuple(2);
-  Field(block,0) = encoder;
+  block = caml_alloc_tuple(2);
+  Store_field(block, 0, encoder);
   /* TODO : clean this */
-  Field(block,1) = ocaml_vorbis_get_header_frame(ve, title, artist, genre, date, album, tracknum);
+  Store_field(block, 1, ocaml_vorbis_get_header_frame(ve, title, artist, genre, date, album, tracknum));
 
   CAMLreturn(block);
 }
@@ -318,87 +313,82 @@ CAMLprim value ocaml_vorbis_create_encoder_byte(value* argv, int argc) {
   return (ocaml_vorbis_create_encoder(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]));
 }
 
-
-/**
- * It does initialize buffer_.
- */
-static int wav_read(myvorbis_encoder_t *ve, char *in, float ***buffer_, int len)
+/* Encode audio from buf, return the available ogg/vorbis data */
+CAMLprim value ocaml_vorbis_encode_buffer(value v_e, value _buf)
 {
-  int sampbyte = ve->in_samplesize / 8;
-  char *buf;
-  int bytes_read;
-  int i, j;
-  int realsamples;
-  float **buffer;
-
-  realsamples = len / (sampbyte * ve->in_channels);
-  *buffer_ = vorbis_analysis_buffer(&ve->vd, realsamples);
-  buffer = *buffer_;
-  bytes_read = realsamples * sampbyte * ve->in_channels;
-  buf = (char*)malloc(bytes_read);
-  memcpy(buf, in, bytes_read);
-
-  if(ve->in_samplesize == 8)
-    {
-      unsigned char *bufu = (unsigned char *)buf;
-      for(i = 0; i < realsamples; i++)
-	{
-	  for(j=0; j < ve->in_channels; j++)
-	    {
-	      buffer[j][i] = ((int)(bufu[i * ve->in_channels + j]) - 128) / 128.0f;
-	    }
-	}
-    }
-  else
-    {
-      if(!ve->in_bigendian)
-	{
-	  for(i = 0; i < realsamples; i++)
-	    {
-	      for(j=0; j < ve->in_channels; j++)
-		{
-		  buffer[j][i] = ((buf[i * 2 * ve->in_channels + 2 * j + 1] << 8) |
-				  (buf[i * 2 * ve->in_channels + 2 * j] & 0xff)) / 32768.0f;
-		}
-	    }
-	}
-      else
-	{
-	  for(i = 0; i < realsamples; i++)
-	    {
-	      for(j=0; j < ve->in_channels; j++)
-		{
-		  buffer[j][i] = ((buf[i * 2 * ve->in_channels + 2 * j] << 8) |
-				  (buf[i * 2 * ve->in_channels + 2 * j + 1] & 0xff)) / 32768.0f;
-		}
-	    }
-	}
-    }
-  free(buf);
-
-  return realsamples;
-}
-
-
-CAMLprim value ocaml_vorbis_encode_buffer(value v_e, value buf)
-{
-  CAMLparam2(v_e, buf);
-
+  CAMLparam2(v_e, _buf);
   myvorbis_encoder_t *ve = ve_of_block(v_e);
-  float **buffer;
-  int samples_read = wav_read(ve, String_val(buf), &buffer, string_length(buf));
+  int len = caml_string_length(_buf);
+  char* in = (char*)malloc(len*sizeof(char));
+  memcpy(in,String_val(_buf),len*sizeof(char));
+
+  /* Ask vorbis the adress where to put the new data.
+   * TODO some data can be lost when downsampling: with input at 22050Hz
+   * and output at 44100Hz, one sample is lost if the input contains an odd
+   * number of samples. It shouldn't hurt too much. */
+  int realsamples =
+    (len * 8 * ve->samplefreq) /
+    (ve->in_samplesize * ve->in_channels * ve->in_samplefreq) ;
+  float** buffer = vorbis_analysis_buffer(&ve->vd, realsamples);
+  int i, j, c;
+  int downsample = ve->in_samplefreq / ve->samplefreq ;
+
+  /* stereo2mono : shall we mix two channels in one ?
+   * Set to 2 if doing so, otherwise it's set to 1. */
+  int s2m = (ve->in_channels==2 && ve->channels==1?2:1);
+
+  /* Then we'll fill [ret] with the new ogg/vorbis data */
   char *ret = NULL;
   int ret_len = 0;
   int eos = 0;
+
+  /* And finally we'll copy it into a caml value */
   CAMLlocal1(s);
 
-  if(samples_read == 0)
-    /* Tell the library that we wrote 0 bytes - signalling the end */
-    vorbis_analysis_wrote(&ve->vd, 0);
-  else
-    /* Tell the library how many samples (per channel) we wrote 
-       into the supplied buffer */
-    vorbis_analysis_wrote(&ve->vd, samples_read);
+  /* For now, we only work on the C side, and let other Caml threads run */
+  caml_enter_blocking_section();
+
+  /* Sample conversion, using the assertion size == 8 or 16.
+   * At the same time we perform downsampling, and channel conversions.
+   * Stereo2mono is made in two passes: on the first one (c=0), we SET
+   * the buffer to half the value of the first channel, on the second
+   * one (c=1) we add half of the second channel.
+   * SELECT(j) is [c] if [s2m], otherwise it's [j], or 0 if [j] is out
+   * of bounds. This last case happens when encoding mono to stereo. */
+#define SET(A,B) (s2m==1?(A=B):(c==0?(A=B/2):(A+=B/2)))
+#define SELECT(A) (s2m==2?c:((A)<ve->in_channels?(A):0))
+  for(c=0;c<s2m;c++)
+    if(ve->in_samplesize == 8)
+    {
+      unsigned char *bufu = (unsigned char *)in;
+      for(i = 0; i < realsamples; i++)
+        for(j = 0; j < ve->channels; j++)
+          SET(buffer[j][i],
+              ((int)(bufu[i * ve->in_channels * downsample
+                     + SELECT(j)]) - 128)
+              / 128.0f);
+    }
+    else
+      if(!ve->in_bigendian)
+        for(i = 0; i < realsamples; i++)
+          for(j = 0; j < ve->channels; j++)
+            SET(buffer[j][i],
+                ((in[i * 2 * ve->in_channels * downsample
+                  + 2 * SELECT(j) + 1] << 8) |
+                 (in[i * 2 * ve->in_channels * downsample
+                  + 2 * SELECT(j)] & 0xff)) / 32768.0f);
+      else
+        for(i = 0; i < realsamples; i++)
+          for(j = 0; j < ve->channels; j++)
+            SET(buffer[j][i],
+                ((in[i * 2 * ve->in_channels * downsample
+                  + 2 * SELECT(j)] << 8) |
+                 (in[i * 2 * ve->in_channels * downsample
+                  + 2 * SELECT(j) + 1] & 0xff)) / 32768.0f);
+
+  /* Tell the library how many samples (per channel) we wrote
+     into the supplied buffer */
+  vorbis_analysis_wrote(&ve->vd, realsamples);
 
   /* While we can get enough data from the library to analyse, one
      block at a time... */
@@ -407,14 +397,14 @@ CAMLprim value ocaml_vorbis_encode_buffer(value v_e, value buf)
       /* Do the main analysis, creating a packet */
       vorbis_analysis(&ve->vb, NULL);
       vorbis_bitrate_addblock(&ve->vb);
-      
-      while(vorbis_bitrate_flushpacket(&ve->vd, &ve->op)) 
+
+      while(vorbis_bitrate_flushpacket(&ve->vd, &ve->op))
 	{
 	  /* Add packet to bitstream */
 	  ogg_stream_packetin(&ve->os, &ve->op);
-	  
+
 	  /* If we've gone over a page boundary, we can do actual output,
-	     so do so (for however many pages are available) */	  
+	     so do so (for however many pages are available) */
 	  while(!eos)
 	    {
 	      if(!ogg_stream_pageout(&ve->os, &ve->og)) break;
@@ -423,14 +413,18 @@ CAMLprim value ocaml_vorbis_encode_buffer(value v_e, value buf)
 	      memcpy(ret + ret_len, (&ve->og)->header, (&ve->og)->header_len);
 	      memcpy(ret + ret_len + (&ve->og)->header_len, (&ve->og)->body, (&ve->og)->body_len);
 	      ret_len += (&ve->og)->header_len + (&ve->og)->body_len;
-	      
+
 	      if(ogg_page_eos(&ve->og)) eos = 1;
 	    }
 	}
     }
 
-  s = copy_buffer(ret, ret_len);
+  /* Back to the Caml world */
+  caml_leave_blocking_section();
+
+  copy_buffer(ret, ret_len, s);
   free(ret);
+  free(in);
 
   CAMLreturn(s);
 }
@@ -464,16 +458,16 @@ static void finalize_myvorbis_dec_file_t(value d_f)
 }
 
 
-CAMLprim value ocaml_vorbis_open_dec_file(value file, value params)
+CAMLprim value ocaml_vorbis_open_dec_file(value fd, value params)
 {
-  CAMLparam2(file, params);
+  CAMLparam2(fd, params);
   CAMLlocal1(block);
 
   FILE *in = NULL;
   myvorbis_dec_file_t *df = (myvorbis_dec_file_t*)malloc(sizeof(myvorbis_dec_file_t));
 
-  if(!(in = fopen(String_val(file), "rb")))
-      raise_constant(*caml_named_value("vorbis_exn_could_not_open_file"));
+  if(!(in = fdopen(Int_val(fd), "rb")))
+      caml_raise_constant(*caml_named_value("vorbis_exn_could_not_open_file"));
   df->ovf = (OggVorbis_File*)malloc(sizeof(OggVorbis_File));
   df->channels = Int_val(Field(params, 0));
   df->sample_freq = Int_val(Field(params, 1));
@@ -484,11 +478,11 @@ CAMLprim value ocaml_vorbis_open_dec_file(value file, value params)
     {
       fclose(in);
       free(df->ovf);
-      raise_constant(*caml_named_value("vorbis_exn_not_a_vorbis_file"));
+      caml_raise_constant(*caml_named_value("vorbis_exn_not_a_vorbis_file"));
     }
 
-  block = alloc_final(2, finalize_myvorbis_dec_file_t, sizeof(myvorbis_dec_file_t), 1000);
-  Field(block, 1) = (value)df;
+  block = caml_alloc_final(2, finalize_myvorbis_dec_file_t, sizeof(myvorbis_dec_file_t), 1000);
+  Store_field(block, 1, (value)df);
   CAMLreturn(block);
 }
 
@@ -504,13 +498,13 @@ CAMLprim value ocaml_vorbis_decode(value d_f, value buf_, value ofs_, value len_
   int len = Int_val(len_);
   char* buf = String_val(buf_) + ofs;
 
-  if (ofs + len > string_length(buf_))
-    raise_constant(*caml_named_value("vorbis_exn_invalid_parameters"));
+  if (ofs + len > caml_string_length(buf_))
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_parameters"));
   ret = ov_read(df->ovf, buf, len, df->big_endian, df->sample_size / 8, df->sign, &bs);
   if(ret == 0)
-      raise_constant(*caml_named_value("exn_end_of_file"));
+    caml_raise_end_of_file ();
   if(ret < 0)
-      raise_constant(*caml_named_value("vorbis_exn_hole_in_data"));
+    caml_raise_constant(*caml_named_value("vorbis_exn_hole_in_data"));
 
   CAMLreturn(Val_int(ret));
 }
@@ -532,9 +526,9 @@ CAMLprim value ocaml_vorbis_utf8_decode(value string)
   char* utf8;
   if(utf8_decode(String_val(string),&utf8) >= 0)
     {
-      CAMLreturn(copy_string(utf8));
+      CAMLreturn(caml_copy_string(utf8));
     }
-  raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), string);
+  caml_raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), string);
 }
 
 CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist, value genre, value date, value album, value tracknum) {
@@ -545,13 +539,13 @@ CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist,
 
   ogg_packet op;
   ogg_page og;
-  CAMLlocal1(string);
+  CAMLlocal1(ans);
   char* ret=NULL;
   int size = 0;
 
   /* tell libvorbis that this is the end of the stream */
   vorbis_analysis_wrote(&s->vd, 0);
-  
+
   while(vorbis_analysis_blockout(&s->vd, &s->vb)==1)
     {
       vorbis_analysis(&s->vb, NULL);
@@ -579,12 +573,12 @@ CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist,
     ogg_packet h1,h2,h3;
 
     build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(date), string_of_option(genre));
-  
+
     comments.vendor = "ocaml-vorbis";
 
     vorbis_analysis_init(&s->vd, &s->vi);
     vorbis_block_init(&s->vd, &s->vb);
-    
+
     ogg_stream_init(&s->os, ++(s->serial));
     vorbis_analysis_headerout(&s->vd, &comments, &h1,&h2,&h3);
     ogg_stream_packetin(&s->os, &h1);
@@ -601,10 +595,10 @@ CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist,
       memcpy(ret + size + (&s->og)->header_len, (&s->og)->body, (&s->og)->body_len);
       size += (&s->og)->header_len + (&s->og)->body_len;
     }
-  
-  string = copy_buffer(ret, size);
+
+  copy_buffer(ret, size, ans);
   free(ret);
-  CAMLreturn(string);
+  CAMLreturn(ans);
 }
 
 value ocaml_vorbis_encoder_reset_byte(value *argv, int argc)
